@@ -1,8 +1,9 @@
-"""Private module for Surface IO in DataIO class."""
+"""Private module for the DataIO class."""
 import json
 import logging
 import warnings
 from collections import OrderedDict
+from copy import deepcopy
 from datetime import datetime
 from pathlib import Path
 from typing import Tuple
@@ -26,12 +27,23 @@ VALID_SURFACE_FORMATS = {"irap_binary": ".gri"}
 VALID_GRID_FORMATS = {"hdf": ".hdf", "roff": ".roff"}
 VALID_CUBE_FORMATS = {"segy": ".segy"}
 VALID_TABLE_FORMATS = {"hdf": ".hdf", "csv": ".csv", "arrow": ".arrow"}
-VALID_POLYGONS_FORMATS = {"hdf": ".hdf", "csv": ".csv", "irap_ascii": ".pol"}
+VALID_POLYGONS_FORMATS = {
+    "hdf": ".hdf",
+    "csv": ".csv",  # columns will be X Y Z, ID
+    "csv|xtgeo": ".csv",  # use default xtgeo columns: X_UTME, Y_UTMN, Z_TVDSS, POLY_ID
+    "irap_ascii": ".pol",
+}
+VALID_POINTS_FORMATS = {
+    "hdf": ".hdf",
+    "csv": ".csv",  # columns will be X Y Z
+    "csv|xtgeo": ".csv",  # use default xtgeo columns: X_UTME, Y_UTMN, Z_TVDSS
+    "irap_ascii": ".poi",
+}
 
+# The produced metadata must conform with the corresponding JSON schema.
+# Metadata definitions are stored under schema/
 
-# the content must conform with the given json schema, e.g.
-# https://github.com/equinor/fmu-metadata/blob/dev/definitions/*/schema/fmu_results.json
-#
+# Some content types have additional input requirements.
 # When value is None, a repeat field shall not be present, otherwise it may be as this:
 # content: seismics
 # seismics:
@@ -50,8 +62,9 @@ ALLOWED_CONTENTS = {
         "zrange": float,
         "filter_size": float,
         "scaling_factor": float,
+        "offset": str,
     },
-    "fluid_contact": {"contact": str},
+    "fluid_contact": {"contact": str, "truncated": bool},
     "field_outline": {"contact": str},
     "regions": None,
     "pinchout": None,
@@ -65,7 +78,7 @@ ALLOWED_CONTENTS = {
 }
 
 # this setting will set if subkeys is required or not. If not found in list then
-# False is assumed.
+# assume False.
 CONTENTS_REQUIRED = {
     "fluid_contact": {"contact": True},
     "field_outline": {"contact": False},
@@ -120,6 +133,7 @@ class _ExportItem:
         dataio,
         obj,
         subfolder=None,
+        forcefolder=None,
         verbosity="WARNING",
         include_index=False,
         name=None,
@@ -140,6 +154,7 @@ class _ExportItem:
         self.display_name = _override_arg(dataio, "display_name", display_name)
         self.unit = _override_arg(dataio, "unit", unit)
         self.subfolder = _override_arg(dataio, "subfolder", subfolder)
+        self.forcefolder = _override_arg(dataio, "forcefolder", forcefolder)
         self.verbosity = _override_arg(dataio, "verbosity", verbosity)
         self.include_index = _override_arg(
             dataio, "include_index", include_index, default=False
@@ -171,25 +186,18 @@ class _ExportItem:
         if self.verbosity is None:
             self.verbosity = "WARNING"  # fallback
 
-        self.realfolder = dataio.realfolder
-        self.iterfolder = dataio.iterfolder
+        self.realname = dataio.realname
+        self.itername = dataio.itername
         self.createfolder = dataio.createfolder
 
-        if subfolder is not None:
-            warnings.warn(
-                "Exporting to a subfolder is a deviation from the standard "
-                "and could have consequences for later dependencies",
-                UserWarning,
-            )
-
     def save_to_file(self) -> str:
-        """Saving (export) an instance to file with rich metadata for SUMO.
+        """Save (export) an instance to file with rich metadata.
 
         Many metadata items are object independent and are treated directly in the
         dataio module. Here additional metadata (dependent on this datatype) are
         collected/processed and subsequently both 'independent' and object dependent
-        a final metadata file (or part of file if HDF) are collected and
-        written to disk here.
+        metadata are collected and written to disk here (as separate file or bundled
+        with data, depending on data type and format).
         """
         logger.info("Save to file...")
         if isinstance(self.obj, xtgeo.RegularSurface):
@@ -204,6 +212,12 @@ class _ExportItem:
             self.efolder = "polygons"
             self.valid = VALID_POLYGONS_FORMATS
             self.fmt = self.dataio.polygons_fformat
+        elif isinstance(self.obj, xtgeo.Points):
+            self.subtype = "Points"
+            self.classname = "points"
+            self.efolder = "points"
+            self.valid = VALID_POINTS_FORMATS
+            self.fmt = self.dataio.points_fformat
         elif isinstance(self.obj, xtgeo.Cube):
             self.subtype = "RegularCube"
             self.classname = "cube"
@@ -242,13 +256,12 @@ class _ExportItem:
 
         self._data_process()
         self._data_process_object()
-        self._fmu_inject_workflow()  # this will vary if surface, table, grid, ...
-        self._display()
+        self._display_process()
         fpath = self._item_to_file()
         return str(fpath)
 
     def _data_process(self):
-        """Process som potentially common subfields in the data block.
+        """Process some potentially common subfields in the data block.
 
         These subfields are:
         - name
@@ -266,12 +279,16 @@ class _ExportItem:
         self._data_process_content()
         self._data_process_parent()
         self._data_process_timedata()
+        self._data_process_description()
+        self._data_process_tagname()
         self._data_process_various()
 
     def _data_process_name(self):
-        """Process the name  and alos the display_name subfield."""
-        # first detect if name is given, or infer name from object if possible
-        # then determine if name is stratgraphic and assing a "true" valid name
+        """Process the name and also the display_name subfield.
+
+        First detect if name is given, or infer name from object if possible
+        then determine if name is stratigraphic and assing a "true" valid name
+        """
         logger.info("Evaluate data:name attribute")
         usename = "unknown"
         meta = self.dataio.metadata4data
@@ -307,9 +324,6 @@ class _ExportItem:
             "Evaluate data:name attribute done, true name is <%s>", meta["name"]
         )
 
-        if self.display_name is None or self.display_name == "unknown":
-            self.display_name = self.name
-
     def _data_process_context(self):
         """Process the context input which gives offset and top/base settings.
 
@@ -388,7 +402,7 @@ class _ExportItem:
             meta[item]["offset"] = offset
 
     def _data_process_content(self):
-        """Process the content block (within data block) which can complex."""
+        """Process the content block (within data block) which can be complex."""
         logger.info("Evaluate content")
         content = self.dataio.content
         logger.debug("content is %s of type %s", str(content), type(content))
@@ -491,12 +505,14 @@ class _ExportItem:
                     "but is not found",
                 )
 
-            # if name in CONTENTS_REQUIRED.keys():
-            #     if key in CONTENTS_REQUIRED[name] and CONTENTS_REQUIRED[name] is True
-
     def _data_process_timedata(self):
-        """Process the time subfield and also contruct self.times."""
+        """Process the time subfield and also construct self.times."""
         # first detect if timedata is given, the process it
+        # timedata may be like:
+        #       None
+        #       [["20220101", "monitor"], ["20200101", "base"]]
+        #       [["20220101", None], ["20200101", None]]
+        #       [["20220101", "any"], None]
         logger.info("Evaluate data:name attribute")
         meta = self.dataio.metadata4data
 
@@ -505,8 +521,20 @@ class _ExportItem:
         if self.timedata is None:
             return
 
+        # this is used in file name construction:
         self.times = []  # e.g. ["20211102", "20231101"] or ["20211102", None]
-        for xtime in self.timedata:
+
+        # normally self.timedata (input) has two entries, but one may accepted,
+        # implicitly meaning the second item is None
+        usetimedata = deepcopy(self.timedata)
+        if len(usetimedata) == 1:
+            usetimedata.append(None)
+
+        for xtime in usetimedata:
+            if xtime is None:
+                self.times.append(None)
+                continue
+
             if isinstance(xtime[0], int):
                 if xtime[0] < datelimits[0] or xtime[0] > datelimits[1]:
                     raise ValidationError(
@@ -533,6 +561,64 @@ class _ExportItem:
                 usetime["label"] = tlabel
             meta["time"].append(usetime)
 
+    def _data_process_description(self):
+        """Process the data.description item.
+
+        Description is described as an array in the schema. But intuitively it is
+        provided as a string. Also need to maintain backwards compatibility for
+        string as input.
+
+        If description is not given, return without action
+        If description is array, stringify all items
+        If description is string, convert to single-item array
+
+        """
+
+        meta = self.dataio.metadata4data
+
+        if self.description is None:
+            return
+
+        if isinstance(self.description, list):
+            meta["description"] = [str(item) for item in self.description]
+        elif isinstance(self.description, str):
+            meta["description"] = [self.description]
+
+    def _data_process_tagname(self):
+        """Process the data.tagname attribute.
+
+        The tagname will be included in outgoing filename if file is exported, and will
+        then be made filename-friendly. The tagname used in the file shall be identical
+        to the one used in data.tagname. Therefore, we will require that incoming
+        tagname is filename friendly.
+
+        For now we will change it with a warning.
+
+        # TODO align this with similar operation in _construct_filename_fmustandard1()
+
+        """
+
+        if self.tagname is None:
+            logger.debug("Incoming tagname is None - returning")
+            return
+
+        meta = self.dataio.metadata4data  # short form
+
+        logger.debug("Incoming tagname is %s", self.tagname)
+
+        tagname = self.tagname.lower().replace(" ", "_").replace(".", "_")
+
+        if tagname != self.tagname:
+            logger.debug("Tagname was changed to %s", tagname)
+            warnings.warn(
+                "'tagname' shall be lowercase and not contain special characters."
+                "The provided tagname will be changed to {tagname}."
+                "In future versions, illegal characters in tagname will not be allowed",
+                UserWarning,
+            )
+
+        meta["tagname"] = tagname
+
     def _data_process_various(self):
         """Process "all the rest" of the generic items.
 
@@ -554,24 +640,11 @@ class _ExportItem:
         meta["is_prediction"] = self.dataio.is_prediction
         meta["is_observation"] = self.dataio.is_observation
 
-        # tmp solution for properties
-        # meta["properties"] = list()
-        # props = OrderedDict()
-        # props["name"] = "SomeName"
-        # props["attribute"] = "SomeAttribute"
-        # props["is_discrete"] = False
-        # props["calculation"] = None
-        # meta["properties"].append(props)
-
         # tmp:
         meta["grid_model"] = None
 
-        # tmp:
-        if self.description is not None:
-            meta["description"] = self.description
-
     def _data_process_object(self):
-        """Process data fileds which are object dependent.
+        """Process data fields which are object dependent.
 
         I.e::
 
@@ -592,6 +665,8 @@ class _ExportItem:
             self._data_process_cpgridproperty()
         elif self.subtype == "Polygons":
             self._data_process_object_polygons()
+        elif self.subtype == "Points":
+            self._data_process_object_points()
         elif self.subtype == "DataFrame":
             self._data_process_object_dataframe()
         elif self.subtype == "ArrowTable":
@@ -744,6 +819,40 @@ class _ExportItem:
         meta["bbox"]["zmax"] = float(zmax)
         logger.info("Process data metadata for Polygons... done!!")
 
+    def _data_process_object_points(self):
+        """Process/collect the data items for Points"""
+        logger.info("Process data metadata for Points")
+
+        dataio = self.dataio
+        poi = self.obj
+
+        meta = dataio.metadata4data
+        # shortform
+        meta["spec"] = OrderedDict()
+
+        # list attributes (extra columns)
+        if len(poi.dataframe.columns) > 3:
+            attrnames = poi.dataframe.columns[3:]
+            meta["spec"]["attributes"] = list(attrnames)
+        meta["spec"]["size"] = int(poi.dataframe.size)
+
+        # min, max:
+        xmin = poi.dataframe[poi.xname].min()
+        xmax = poi.dataframe[poi.xname].max()
+        ymin = poi.dataframe[poi.yname].min()
+        ymax = poi.dataframe[poi.yname].max()
+        zmin = poi.dataframe[poi.zname].min()
+        zmax = poi.dataframe[poi.zname].max()
+
+        meta["bbox"] = OrderedDict()
+        meta["bbox"]["xmin"] = float(xmin)
+        meta["bbox"]["xmax"] = float(xmax)
+        meta["bbox"]["ymin"] = float(ymin)
+        meta["bbox"]["ymax"] = float(ymax)
+        meta["bbox"]["zmin"] = float(zmin)
+        meta["bbox"]["zmax"] = float(zmax)
+        logger.info("Process data metadata for Points... done!!")
+
     def _data_process_object_dataframe(self):
         """Process/collect the data items for DataFrame."""
         logger.info("Process data metadata for DataFrame (tables)")
@@ -781,11 +890,7 @@ class _ExportItem:
         meta["bbox"] = None
         logger.info("Process data metadata for ArrowTable... done!!")
 
-    def _fmu_inject_workflow(self):
-        """Inject workflow into fmu metadata block."""
-        self.dataio.metadata4fmu["workflow"] = self.dataio.workflow
-
-    def _display(self):
+    def _display_process(self):
         """Process common subfields in the display block.
 
         For now, this is simply injecting a skeleton with loose
@@ -810,30 +915,23 @@ class _ExportItem:
         """
 
         logger.info("Processing display")
+        self._display_process_name()
+        logger.info("Display has been processed.")
+
+    def _display_process_name(self):
+        """Process display.name"""
+
+        logger.info("Processing display name")
         meta = self.dataio.metadata4display
 
-        # display.name
-        if self.display_name is not None:
-            # first choice: display_name argument
-            logger.debug("display.name is set from arguments")
-            meta["name"] = self.display_name
-        elif self.name is not None:
-            # second choice: name argument
-            logger.debug("display.name is set to name argument as fallback")
-            meta["name"] = self.name
-        else:
-            # third choice: object name, unless the XTgeo default "unknown"
-            try:
-                meta["name"] = self.obj.name
-                logger.debug("display.name is set to object name as fallback")
+        logger.debug("self.display_name is %s", self.display_name)
+        logger.debug("self.name is %s", self.name)
 
-                if meta["name"] == "unknown":
-                    logger.debug("Got default object name from XTgeo, changing to None")
-                    meta["name"] = None
-            except AttributeError:
-                logger.debug("display.name could not be set")
-                meta["name"] = None
+        meta["name"] = (
+            self.display_name or self.name or _utils.get_object_name(self.obj)
+        )
 
+        logger.info("display.name is set to %s", meta["name"])
         logger.info("Processing display is done!")
 
     def _item_to_file(self):
@@ -933,6 +1031,10 @@ class _ExportItem:
 
         dest = outroot / loc
 
+        # override in case of forcefolder!
+        if self.forcefolder:
+            dest = self.forcefolder
+
         if self.subfolder:
             dest = dest / self.subfolder
 
@@ -970,14 +1072,16 @@ class _ExportItem:
             (Path(filepath) / ("." + filestem.lower())).with_suffix(ext + ".yml")
         ).resolve()
 
-        # get the relative path (relative to runptah if interactive, and to casedir
+        # get the relative path (relative to runpath if interactive, and to casedir
         # if this is an ERT run)
 
         useroot = self.dataio.runpath.resolve()
         logger.info("The useroot (initial) is %s", useroot)
-        if self.iterfolder:
+        if self.itername:
             useroot = (useroot / "../..").resolve()
             logger.info("The useroot (updated) is %s", useroot)
+        else:
+            logger.info("No itername, so not updating useroot")
 
         relpath = abspath.relative_to(useroot)
 
@@ -1002,14 +1106,34 @@ class _ExportItem:
             self.obj.to_file(outfile, fformat="roff")
             self.dataio.metadata4data["format"] = "roff"
         elif "csv" in self.fmt and self.subtype == "Polygons":
-            renamings = {"X_UTME": "X", "Y_UTMN": "Y", "Z_TVDSS": "Z", "POLY_ID": "ID"}
             worker = self.obj.dataframe.copy()
-            worker.rename(columns=renamings, inplace=True)
+            if "xtgeo" not in self.fmt:
+                renamings = {
+                    self.obj.xname: "X",
+                    self.obj.yname: "Y",
+                    self.obj.zname: "Z",
+                    self.obj.pname: "ID",
+                }
+                worker.rename(columns=renamings, inplace=True)
+            worker.to_csv(outfile, index=False)
+            self.dataio.metadata4data["format"] = "csv"
+        elif "csv" in self.fmt and self.subtype == "Points":
+            worker = self.obj.dataframe.copy()
+            if "xtgeo" not in self.fmt:
+                renamings = {
+                    self.obj.xname: "X",
+                    self.obj.yname: "Y",
+                    self.obj.zname: "Z",
+                }
+                worker.rename(columns=renamings, inplace=True)
             worker.to_csv(outfile, index=False)
             self.dataio.metadata4data["format"] = "csv"
         elif "irap_ascii" in self.fmt and self.subtype == "Polygons":
             self.obj.to_file(outfile)
             self.dataio.metadata4data["format"] = "irap_ascii"
+        elif "irap_ascii" in self.fmt and self.subtype == "Points":
+            self.obj.to_file(outfile)
+            self.dataio.metadata4data["format"] = "irap_ascii"
         elif self.fmt == "csv" and self.subtype == "DataFrame":
             logger.info("Exporting table as csv, with INDEX %s", self.include_index)
             self.obj.to_csv(outfile, index=self.include_index)
