@@ -6,6 +6,8 @@ import uuid
 from pathlib import Path
 from typing import Dict, List, Union
 
+import xtgeo
+
 from . import _design_kw
 from . import _oyaml as oyaml
 
@@ -153,11 +155,6 @@ def nested_parameters_dict(
             unique_keys.append(key)
             nested_dict[key] = value
 
-    if len(unique_keys) > len(set(unique_keys)):
-        raise ValueError(
-            "Keys in parameters dictionary are not unique after removal of namespace"
-        )
-
     return nested_dict
 
 
@@ -177,3 +174,36 @@ def check_if_number(value):
         return res
 
     return value
+
+
+def get_object_name(obj):
+    """Get the name of the object.
+
+    If not possible, return None.
+    If result is 'unknown', return None (XTgeo defaults)
+    If object is a polygon, and object name is 'poly', return None (XTgeo defaults)
+    If object is a grid, and object name is 'noname', return None (XTgeo defaults)
+
+    """
+
+    logger.debug("Getting name from the data object itself")
+
+    try:
+        name = obj.name
+    except AttributeError:
+        logger.info("display.name could not be set")
+        return
+
+    if isinstance(obj, xtgeo.RegularSurface) and name == "unknown":
+        logger.debug("Got 'unknown' as name from a surface object, returning None")
+        return
+
+    if isinstance(obj, xtgeo.Polygons) and name == "poly":
+        logger.debug("Got 'poly' as name from a polygons object, returning None")
+        return
+
+    if isinstance(obj, xtgeo.Grid) and name == "noname":
+        logger.debug("Got 'noname' as name from grids object, returning None")
+        return
+
+    return name
