@@ -3,6 +3,7 @@ import logging
 import shutil
 from collections import OrderedDict
 
+import pandas as pd
 import xtgeo
 import yaml
 
@@ -22,6 +23,11 @@ CFG["masterdata"] = {
         "discovery": [{"short_identifier": "abdcef", "uuid": "ghijk"}],
     }
 }
+CFG["access"] = {
+    "asset": "Drogon",
+    "ssdl": {"access_level": "internal", "some_access_tag": True},
+}
+CFG["model"] = {"revision": "0.99.0"}
 
 CFG2 = {}
 with open("tests/data/drogon/global_config2/global_variables.yml", "r") as stream:
@@ -49,11 +55,33 @@ def test_polygons_io(tmp_path):
     srf = xtgeo.Polygons(POLY)
     fmu.dataio.ExportData.polygons_fformat = "csv"
 
-    exp = fmu.dataio.ExportData(name="test", content="depth", runpath=tmp_path)
+    exp = fmu.dataio.ExportData(
+        name="test", content="depth", runpath=tmp_path, config=CFG
+    )
     exp.export(srf)
 
     assert (tmp_path / FMUP1 / "polygons" / ".test.csv.yml").is_file() is True
 
+    thedataframe = pd.read_csv(tmp_path / FMUP1 / "polygons" / "test.csv")
+    assert list(thedataframe.columns) == ["X", "Y", "Z", "ID"]
+
+
+def test_polygons_io_xtgeo_csv(tmp_path):
+    """Minimal test polygons io, uses csv with xtgeo column names"""
+
+    srf = xtgeo.Polygons(POLY)
+    fmu.dataio.ExportData.polygons_fformat = "csv|xtgeo"
+
+    exp = fmu.dataio.ExportData(
+        name="test99", content="depth", runpath=tmp_path, config=CFG
+    )
+    exp.export(srf)
+
+    assert (tmp_path / FMUP1 / "polygons" / ".test99.csv.yml").is_file() is True
+
+    thedataframe = pd.read_csv(tmp_path / FMUP1 / "polygons" / "test99.csv")
+    assert list(thedataframe.columns) == ["X_UTME", "Y_UTMN", "Z_TVDSS", "POLY_ID"]
+
 
 def test_polygons_io_larger_case_ertrun(tmp_path):
     """Larger test polygons io as ERTRUN, uses global config from Drogon to tmp_path."""
@@ -94,8 +122,7 @@ def test_polygons_io_larger_case_ertrun(tmp_path):
     )
 
     # make a fake Polygons object
-    poly = xtgeo.Polygons()
-    poly.from_list([(123.0, 345.0, 222.0, 0), (123.0, 345.0, 222.0, 0)])
+    poly = xtgeo.Polygons([(123.0, 345.0, 222.0, 0), (123.0, 345.0, 222.0, 0)])
     print(poly.dataframe)
 
     exp.export(poly, verbosity="INFO")
